<?php
/**
 * @file
 * Hooks provided by Commerce Booking.
 */

/**
 * @addtogroup hooks
 * @{
 */

/**
 * Influence whether a booking can be created.
 *
 * This allows modules to deny a user from creating a booking. Examples would
 * be a module that links tickets to users and therefore wants to prevent
 * ticket holders booking again or a module that tracks capacity of an event.
 *
 * If any module returns FALSE, a booking will be denied.
 *
 * @param string $entity_type
 *   The entity type of the even entity.
 * @param Object $entity
 *   The event entity.
 * @param stdClass $account
 *   The account to prepare the booking for.
 *
 * @return NULL|FALSE
 *   Return FALSE to deny an order being created.
 *
 * @see commerce_booking_prepare_booking()
 */
function hook_commerce_booking_allow_booking($entity_type, $entity, $account) {
  // Check whether there is capacity for more bookings on this event.
  if (mymodule_event_is_full($entity_type, $entity)) {
    return FALSE;
  }
}

/**
 * Define Commerce Booking Ticket statuses.
 *
 * Commerce Booking defines the following statuses by default:
 * - Pending: The ticket has been requested (added to a booking) but no deposit
 *   has been paid yet.
 * - Deposit Paid: The deposit has been paid on this ticket.
 * - Paid in Full: This ticket is confirmed and has been paid for in full.
 * - Cancelled: The ticket has been cancelled.
 *
 * @return array
 *   An array of status arrays keyed by name.
 */
function hook_commerce_booking_ticket_status_info() {
  $statuses = array();

  $statuses['pending'] = array(
    'name' => 'pending',
    'label' => t('Pending'),
    'description' => t('The deposit has not yet been paid on this ticket.'),
    'default' => TRUE,
  );

  $statuses['deposit_paid'] = array(
    'name' => 'deposit_paid',
    'label' => t('Deposit Paid'),
    'description' => t('The deposit has paid on this ticket.'),
  );

  $statuses['paid_in_full'] = array(
    'name' => 'paid_in_full',
    'label' => t('Paid in Full'),
    'description' => t('This ticket has been paid for in full.'),
  );

  $statuses['cancelled'] = array(
    'name' => 'cancelled',
    'label' => t('Cancelled'),
    'description' => t('This ticket has been cancelled.'),
  );

  return $statuses;
}

/**
 * Allow modules to alter Commerce Booking Ticket statuses.
 *
 * @param array $statuses
 *   An array of Status arrays keyed by name.
 *
 * @see hook_commerce_booking_ticket_status_info()
 */
function hook_commerce_bookng_ticket_status_info_alter(&$statuses) {
  // No example.
}

/**
 * @} End of "addtogroup hooks".
 */
