<?php
/**
 * @file
 * Contains Drupal Commerce hooks.
 */

/**
 * Implements hook_commerce_checkout_page_info().
 */
function commerce_booking_commerce_checkout_page_info() {
  $pages = array();

  $pages['terms_conditions'] = array(
    'page_id' => 'terms_conditions',
    'title' => t('Terms & Conditions'),
    'weight' => -99,
    'status_cart' => FALSE,
  );

  $pages['tickets'] = array(
    'page_id' => 'tickets',
    'title' => t('Tickets'),
    'weight' => 1,
    'status_cart' => FALSE,
  );

  return $pages;
}

/**
 * Implements hook_commerce_checkout_pane_info().
 */
function commerce_booking_commerce_checkout_pane_info() {
  $panes = array();

  $panes['terms_conditions'] = array(
    'pane_id' => 'terms_conditions',
    'title' => FALSE,
    'name' => t('Terms & Conditions'),
    'page' => 'terms_conditions',
    'review' => FALSE,
    'file' => 'commerce_booking.checkout.inc',
    'base' => 'commerce_booking_checkout_pane_terms_conditions',
  );

  $panes['tickets'] = array(
    'pane_id' => 'tickets',
    'title' => t('Tickets'),
    'name' => t('Tickets'),
    'page' => 'tickets',
    'file' => 'commerce_booking.checkout.inc',
    'base' => 'commerce_booking_checkout_pane_tickets',
  );

  $panes['ticket_payments'] = array(
    'pane_id' => 'ticket_payments',
    'title' => t('Tickets'),
    'name' => t('Ticket Payments'),
    'page' => 'review',
    'file' => 'commerce_booking.checkout.inc',
    'base' => 'commerce_booking_checkout_pane_ticket_payments',
  );

  return $panes;
}

/**
 * Implements hook_commerce_line_item_type_info().
 */
function commerce_booking_commerce_line_item_type_info() {
  $line_item_types = array();

  $line_item_types['commerce_booking_ticket'] = array(
    'type' => 'commerce_booking_ticket',
    'name' => t('Ticket'),
    'description' => t('References a ticket and event.'),
    'product' => TRUE,
    'add_form_submit_value' => t('Add ticket'),
    'callbacks' => array(
      'title' => 'commerce_booking_line_item_ticket_title',
    ),
  );

  return $line_item_types;
}

/**
 * Title callback for commerce_booking_ticket line item.
 */
function commerce_booking_line_item_ticket_title($line_item) {
  $wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
  $num_tickets = 0;
  foreach ($wrapper->commerce_booking_ticket as $ticket_wrapper) {
    $num_tickets++;
  }

  $event_label = $wrapper->order->commerce_booking_event->label();
  return $num_tickets ? t('@num tickets for @event', array('@num' => $num_tickets, '@event' => $event_label)) : t('Invalid Ticket');
}

/**
 * Calculate the value of a line item.
 *
 * @param stdClass $line_item
 *   The line item to calculate the value of.
 */
function commerce_booking_line_item_ticket_process(&$line_item) {
  if ($line_item->type != 'commerce_booking_ticket') {
    drupal_set_message('Invalid line item type.', 'error');
    return;
  }

  $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);

  // Sum the ticket prices.
  $total_amount = 0;
  foreach ($line_item_wrapper->commerce_booking_ticket as $ticket_wrapper) {
    if ($ticket_wrapper->value()) {
      $price = $ticket_wrapper->commerce_booking_ticket_price->value();
      $total_amount += $price['amount'];

      // If one of the tickets has no value set stop us from losing currency
      // information.
      if (!empty($price['currency_code'])) {
        $currency = $price['currency_code'];
      }
    }
    else {
      watchdog('commerce_booking', 'Ticket %ticket_id not found for order %order_id.', array(
        '%ticket_id' => $ticket_wrapper->getIdentifier(),
        '%order_id' => $line_item_wrapper->order_id->value(),
      ), WATCHDOG_WARNING);
    }
  }

  $line_item->commerce_unit_price[LANGUAGE_NONE][0]
    = $line_item->commerce_total[LANGUAGE_NONE][0] = array(
      'amount' => $total_amount,
      'currency_code' => isset($currency) ? $currency : commerce_default_currency(),
    );

  if (!is_null($line_item_wrapper->commerce_unit_price->value())) {
    // Add the base price to the components array.
    if (!commerce_price_component_load($line_item_wrapper->commerce_unit_price->value(), 'base_price')) {
      $line_item_wrapper->commerce_unit_price->data = commerce_price_component_add(
        $line_item_wrapper->commerce_unit_price->value(),
        'base_price',
        $line_item_wrapper->commerce_unit_price->value(),
        TRUE
      );
    }
  }

  if (!is_null($line_item_wrapper->commerce_total->value())) {
    // Add the base price to the components array.
    if (!commerce_price_component_load($line_item_wrapper->commerce_total->value(), 'base_price')) {
      $line_item_wrapper->commerce_total->data = commerce_price_component_add(
        $line_item_wrapper->commerce_total->value(),
        'base_price',
        $line_item_wrapper->commerce_total->value(),
        TRUE
      );
    }
  }
}

/**
 * Implements hook_commerce_checkout_complete().
 */
function commerce_booking_commerce_checkout_complete($order) {
  // Only act on commerce booking orders.
  if ($order->type != 'commerce_booking') {
    return;
  }

  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);

  $order_balance = commerce_payment_order_balance($order);

  foreach ($order_wrapper->commerce_line_items as $line_item_wrapper) {
    foreach ($line_item_wrapper->commerce_booking_ticket as $ticket_wrapper) {
      if ($order_balance['amount'] == 0 && $ticket_wrapper->status->value() != 'cancelled') {
        // This ticket is paid in full.
        $ticket_wrapper->status = 'paid_in_full';
      }
      elseif (in_array($ticket_wrapper->status->value(), array('pending', 'deposit_paid'))) {
        $ticket_wrapper->status = 'deposit_paid';
      }

      $ticket_wrapper->save();
    }
  }
}
