<?php
/**
 * @file
 * Page callbacks for commerce booking.
 */

/**
 * Page callback for editing tickets.
 */
function commerce_booking_edit_ticket($js, $ticket = NULL, $order = NULL) {
  if (!$ticket && !$order) {
    return MENU_NOT_FOUND;
  }

  if ($js) {
    // This is in a modal, so let's do the modal magic.
    ctools_include('ajax');
    ctools_include('modal');

    $form_state = array(
      'ajax' => TRUE,
      'build_info' => array(
        'args' => array($ticket, $order),
      ),
    );
    $commands = ctools_modal_form_wrapper('commerce_booking_edit_ticket_form', $form_state);

    if (!empty($form_state['executed'])) {
      // The form has been executed, so let's redirect to the destination page.
      $commands = array();
      $commands[] = ajax_command_invoke('#refresh-ticket-form', 'click');
      $commands[] = ctools_modal_command_dismiss();
    }

    print ajax_render($commands);
    ajax_footer();
    return;
  }
  else {
    // This is just a page, so we don't need to worry.
    return drupal_get_form('commerce_booking_edit_ticket_form', $ticket, $order);
  }
}

/**
 * Form constructor for the edit ticket form.
 *
 * @param CommerceBookingTicket $ticket
 *   A commerce_booking_ticket if we are editing one.
 * @param stdClass $order
 *   A commerce_order if we are adding a ticket.
 *
 * @see commerce_booking_edit_ticket_form_validate()
 * @see commerce_booking_edit_ticket_form_submit()
 */
function commerce_booking_edit_ticket_form($form, &$form_state, $ticket, $order) {
  // If we are adding a ticket, let's create it.
  if ($order && !$ticket) {
    $ticket = commerce_booking_ticket_create($order);
  }
  // If we are editing, let's pull through the order.
  elseif ($ticket && !$order) {
    if ($line_item = commerce_booking_get_line_item($ticket)) {
      $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
      $order = $line_item_wrapper->order->value();
    }
  }

  // If we haven't managed to figure out both, we can't continue.
  if (!$ticket || !$order) {
    return drupal_not_found();
  }

  $form['#ticket'] = $ticket;
  $form['#order']
    = $form_state['order'] = $order;

  // Attach the ticket form.
  field_attach_form('commerce_booking_ticket', $ticket, $form, $form_state);

  // Hide form field we don't want people to edit.
  $form['redirect']['#access'] = FALSE;
  $form['commerce_booking_event']['#access'] = FALSE;
  $form['party']['#access'] = FALSE;
  $form['commerce_booking_ticket_price']['#access'] = FALSE;
  $form['field_ticket_holder_name'][LANGUAGE_NONE][0]['#title_display'] = 'invisible';

  // Add our submit handlers.
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => !empty($ticket->is_new) ? t('Add Ticket') : t('Update ticket'),
  );
  $form['actions']['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
    '#name' => 'op',
    '#limit_validation_errors' => array(),
    '#validate' => array(),
    '#submit' => array(),
  );

  return $form;
}

/**
 * Form validation handler for commerce_booking_edit_ticket_form().
 *
 * @see commerce_booking_edit_ticket_form_submit()
 */
function commerce_booking_edit_ticket_form_validate(&$form, &$form_state) {
  field_attach_form_validate('commerce_booking_ticket', $form['#ticket'], $form, $form_state);

  // Check no other tickets with this email exist on this booking.
  $order_wrapper = entity_metadata_wrapper('commerce_order', $form['#order']);
  $this_ticket_wrapper = entity_metadata_wrapper('commerce_booking_ticket', $form['#ticket']);

  // If no email is provided, we don't want to do this.
  if ($this_ticket_wrapper->getPropertyInfo('ticket_holder_email')
    && $this_ticket_wrapper->ticket_holder_email->value()) {
    foreach ($order_wrapper->commerce_line_items as $line_item_wrapper) {
      foreach ($line_item_wrapper->commerce_booking_ticket as $ticket_wrapper) {
        if ($ticket_wrapper->getIdentifier() == $this_ticket_wrapper->getIdentifier()) {
          continue;
        }

        if ($ticket_wrapper->ticket_holder_email->value() == $this_ticket_wrapper->ticket_holder_email->value()) {
          form_set_error('ticket_holder_email', t('A Ticket with this e-mail has already been added to your booking, if you would like to continue with this ticket please change or remove the e-mail'));
          break;
        }
      }
    }
  }
}

/**
 * Form submission handler for commerce_booking_edit_ticket_form().
 *
 * @see commerce_booking_edit_ticket_form_validate()
 */
function commerce_booking_edit_ticket_form_submit(&$form, &$form_state) {
  $is_new = empty($form['#ticket']->ticket_id);
  $order_wrapper = entity_metadata_wrapper("commerce_order", $form['#order']);

  field_attach_submit('commerce_booking_ticket', $form['#ticket'], $form, $form_state);
  entity_save('commerce_booking_ticket', $form['#ticket']);

  if ($is_new) {
    module_load_include('inc', 'commerce_booking', 'commerce_booking.commerce');

    foreach ($order_wrapper->commerce_line_items as $wrapper) {
      if ($wrapper->getBundle() == 'commerce_booking_ticket') {
        $line_item_wrapper = $wrapper;
        break;
      }
    }

    $line_item_wrapper->commerce_booking_ticket[] = $form['#ticket'];
    commerce_booking_ticket_process($form['#ticket']);
    commerce_booking_line_item_ticket_process($line_item_wrapper->value(), $form['#order']);
    $line_item_wrapper->save();

  }

  // Make sure the order gets updated.
  $order_wrapper->save();
}
